import { MappingAlgorithm, theme } from 'antd'
import { uniq } from 'lodash-es'
import { create } from 'zustand'

type ConfigState = {
  themeConfig: {
    algorithm: MappingAlgorithm[],
    _algorithm: string[],
    primaryColor: string
  },
  themeOptions: { label: string, value: string }[],
}

type Actions = {
  setAlgorithm: (v: string) => void
  setCompactAlgorithm: (v: string) => void
}

const algorithmMap: Record<string, MappingAlgorithm> = {
  default: theme.defaultAlgorithm,
  dark: theme.darkAlgorithm,
  compact: theme.compactAlgorithm
}

// Get theme from localStorage or use default
const getStoredTheme = () => {
  const storedTheme = localStorage.getItem('theme_mode')
  return storedTheme && ['default', 'dark', 'compact'].includes(storedTheme) 
    ? storedTheme 
    : 'default'
}

// Initialize HTML class based on theme
const initThemeClass = (themeName: string) => {
  if (themeName === 'dark') {
    document.documentElement.classList.add('dark')
  } else {
    document.documentElement.classList.remove('dark')
  }
}

// Initialize theme from localStorage
const initialTheme = getStoredTheme()
initThemeClass(initialTheme)

const useConfigStore = create<ConfigState & Actions>()((set) => ({
  themeConfig: {
    _algorithm: initialTheme === 'default' ? ['default'] : [initialTheme],
    algorithm: initialTheme === 'default' ? [theme.defaultAlgorithm] : [algorithmMap[initialTheme]],
    primaryColor: '#1890ff'
  },
  themeOptions: [
    { label: 'Light', value: 'default' },
    { label: 'Dark', value: 'dark' },
    { label: 'Compact', value: 'compact' },
  ],
  setAlgorithm: (v: string) => {
    // Store theme in localStorage
    localStorage.setItem('theme_mode', v)
    
    // Update HTML class for dark mode
    if (v === 'dark') {
      document.documentElement.classList.add('dark')
    } else {
      document.documentElement.classList.remove('dark')
    }
    
    set((state) => {
      const includesCompact = state.themeConfig._algorithm.includes('compact')
      const _algorithm = includesCompact ? [v, 'compact'] : [v]

      return {
        themeConfig: {
          _algorithm,
          algorithm: _algorithm.map((item) => algorithmMap[item]),
          primaryColor: state.themeConfig.primaryColor
        }
      }
    })
  },
  setCompactAlgorithm: (v: string) => {
    set((state) => {
      const withoutCompact = state.themeConfig._algorithm.filter((item) => item !== 'compact')
      const _algorithm = uniq(v ? [...withoutCompact, 'compact'] : withoutCompact)

      return ({
        themeConfig: {
          _algorithm,
          algorithm: _algorithm.map((item) => algorithmMap[item]),
          primaryColor: state.themeConfig.primaryColor
        }
      })
    })
  }
}))

export default useConfigStore
